#!/usr/bin/python3
import subprocess
import argparse
import sys
from pathlib import Path

parser = argparse.ArgumentParser()
parser.add_argument('project')
parser.add_argument('-r', '--release', action='store_true', help='builds in optimized release mode')
parser.add_argument('-e', '--example', action='store_true', help='choose the example to build')

namespace = parser.parse_args()

project_name = namespace.project
example_folder = "./examples/%s"%project_name

build_flags = ['-debug', '-vet']
if namespace.release:
	build_flags = ['-o:speed', '-vet', '-disable-assert']

out = '-out:%s'%project_name
if sys.platform.startswith('win'):
	out += '.exe'
else:
	out += '.bin'

for child in Path(example_folder).iterdir():
	extension = child.suffix
	if child.is_file() and extension in ['.vert', '.frag']:
		subprocess.check_call(['glslc', child, '-o', '%s/%s.spv'%(example_folder, child.name)])

subprocess.check_call(['odin', 'build', example_folder, out] + build_flags)
