package tlr

import "xcb"
import "xcb/icccm"
import "core:c"
import "core:fmt"

main :: proc() {
	screen_number: c.int
	connection := xcb.connect(nil, &screen_number)
	defer xcb.disconnect(connection)
	assert(xcb.connection_has_error(connection) == 0)

	setup := xcb.get_setup(connection)
	screens_iter := xcb.setup_roots_iterator(setup)
	screen: ^xcb.screen_t = &screens_iter.data[screen_number]
	winodw_resizable: b32 = false

	window := xcb.generate_id(connection)

	event_mask: c.int = c.int(xcb.event_mask_t.EXPOSURE)|c.int(xcb.event_mask_t.KEY_PRESS)|c.int(xcb.event_mask_t.KEY_RELEASE)
	xcb.create_window(
		c = connection,
		depth = screen.root_depth,
		wid = window,
		parent = screen.root,
		x = 0,
		y = 0,
		width = 500,
		height = 500,
		border_width = 0,
		_class = cast(u16)xcb.window_class_t.INPUT_OUTPUT,
		visual = screen.root_visual,
		value_mask = cast(u32)xcb.cw_t.EVENT_MASK,
		value_list = &event_mask)

	window_title: cstring = "Tailor Test"
	icccm.set_wm_name(connection, window, cast(u32)xcb.atom_enum_t.STRING, 8, cast(u32)len(window_title), window_title)

	if !winodw_resizable {
		size_hints: icccm.size_hints_t
		icccm.size_hints_set_min_size(&size_hints, 500, 500)
		icccm.size_hints_set_max_size(&size_hints, 500, 500)
		icccm.set_wm_size_hints(connection, window, cast(u32)xcb.atom_enum_t.WM_NORMAL_HINTS, &size_hints)
	}

	xcb.map_window(connection, window)
	flush_res := xcb.flush(connection)
	assert(flush_res > 0)
	
	fmt.println("Expose: ", xcb.EXPOSE)
	for {
		for event in xcb.iter_events(connection) {
			#partial switch event in event {
			case (^xcb.key_press_event_t):
				fmt.println(event.detail)
			}
		}
	}
}
