package xcb

import ctypes "core:c"

EVENT_RESPONSE_TYPE_MASK_INV :: ~u8(0x80)

event_response_type :: #force_inline proc(response_type: u8) -> u8 {
	return response_type & EVENT_RESPONSE_TYPE_MASK_INV
}

Event :: union {
	^dri2_buffer_swap_complete_event_t,
	^dri2_invalidate_buffers_event_t,
	^key_press_event_t,
	^button_press_event_t,
	^motion_notify_event_t,
	^enter_notify_event_t,
	^focus_in_event_t,
	^keymap_notify_event_t,
	^expose_event_t,
	^graphics_exposure_event_t,
	^no_exposure_event_t,
	^visibility_notify_event_t,
	^create_notify_event_t,
	^destroy_notify_event_t,
	^unmap_notify_event_t,
	^map_notify_event_t,
	^map_request_event_t,
	^reparent_notify_event_t,
	^configure_notify_event_t,
	^configure_request_event_t,
	^gravity_notify_event_t,
	^resize_request_event_t,
	^circulate_notify_event_t,
	^property_notify_event_t,
	^selection_clear_event_t,
	^selection_request_event_t,
	^selection_notify_event_t,
	^colormap_notify_event_t,
	^client_message_event_t,
	^mapping_notify_event_t,
	^ge_generic_event_t,
	^key_release_event_t,
	^button_release_event_t,
	^leave_notify_event_t,
	^focus_out_event_t,
	^circulate_request_event_t,
}


iter_events :: proc(c: ^connection_t) -> (e: Event, has_next: bool) {
	generic_event := poll_for_event(c)
	if generic_event == nil {
		has_next = false
		return
	}

	switch event_response_type(generic_event.response_type) {
	case DRI2_BUFFER_SWAP_COMPLETE: e = transmute(^dri2_buffer_swap_complete_event_t)generic_event
	case DRI2_INVALIDATE_BUFFERS: e = transmute(^dri2_invalidate_buffers_event_t)generic_event
	case KEY_PRESS: e = transmute(^key_press_event_t)generic_event
	case BUTTON_PRESS: e = transmute(^button_press_event_t)generic_event
	case MOTION_NOTIFY: e = transmute(^motion_notify_event_t)generic_event
	case ENTER_NOTIFY: e = transmute(^enter_notify_event_t)generic_event
	case FOCUS_IN: e = transmute(^focus_in_event_t)generic_event
	case KEYMAP_NOTIFY: e = transmute(^keymap_notify_event_t)generic_event
	case EXPOSE: e = transmute(^expose_event_t)generic_event
	case GRAPHICS_EXPOSURE: e = transmute(^graphics_exposure_event_t)generic_event
	case NO_EXPOSURE: e = transmute(^no_exposure_event_t)generic_event
	case VISIBILITY_NOTIFY: e = transmute(^visibility_notify_event_t)generic_event
	case CREATE_NOTIFY: e = transmute(^create_notify_event_t)generic_event
	case DESTROY_NOTIFY: e = transmute(^destroy_notify_event_t)generic_event
	case UNMAP_NOTIFY: e = transmute(^unmap_notify_event_t)generic_event
	case MAP_NOTIFY: e = transmute(^map_notify_event_t)generic_event
	case MAP_REQUEST: e = transmute(^map_request_event_t)generic_event
	case REPARENT_NOTIFY: e = transmute(^reparent_notify_event_t)generic_event
	case CONFIGURE_NOTIFY: e = transmute(^configure_notify_event_t)generic_event
	case CONFIGURE_REQUEST: e = transmute(^configure_request_event_t)generic_event
	case GRAVITY_NOTIFY: e = transmute(^gravity_notify_event_t)generic_event
	case RESIZE_REQUEST: e = transmute(^resize_request_event_t)generic_event
	case CIRCULATE_NOTIFY: e = transmute(^circulate_notify_event_t)generic_event
	case PROPERTY_NOTIFY: e = transmute(^property_notify_event_t)generic_event
	case SELECTION_CLEAR: e = transmute(^selection_clear_event_t)generic_event
	case SELECTION_REQUEST: e = transmute(^selection_request_event_t)generic_event
	case SELECTION_NOTIFY: e = transmute(^selection_notify_event_t)generic_event
	case COLORMAP_NOTIFY: e = transmute(^colormap_notify_event_t)generic_event
	case CLIENT_MESSAGE: e = transmute(^client_message_event_t)generic_event
	case MAPPING_NOTIFY: e = transmute(^mapping_notify_event_t)generic_event
	case GE_GENERIC: e = transmute(^ge_generic_event_t)generic_event
	case KEY_RELEASE: e = transmute(^key_release_event_t)generic_event
	case BUTTON_RELEASE: e = transmute(^button_release_event_t)generic_event
	case LEAVE_NOTIFY: e = transmute(^leave_notify_event_t)generic_event
	case FOCUS_OUT: e = transmute(^focus_out_event_t)generic_event
	case CIRCULATE_REQUEST: e = transmute(^circulate_request_event_t)generic_event
	case: e = nil
	}
	has_next = true
	return
}
